@@ -2,7 +2,7 @@
  *
  * GNOME Power Manager Inhibit Applet
  * Copyright (C) 2006 Benjamin Canou <bookeldor@gmail.com>
- * Copyright (C) 2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2006-2007 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -31,9 +31,9 @@
 #include <gtk/gtk.h>
 #include <gtk/gtkbox.h>
 #include <libgnomeui/gnome-help.h>
+#include <libgpm.h>
 
 #include "inhibit-applet.h"
-#include "../../src/gpm-common.h"
 
 static void      gpm_inhibit_applet_class_init (GpmInhibitAppletClass *klass);
 static void      gpm_inhibit_applet_init       (GpmInhibitApplet *applet);
@@ -279,7 +279,7 @@ dialog_about_cb (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 	about = (GtkAboutDialog*) gtk_about_dialog_new ();
 	gtk_about_dialog_set_name (about, GPM_INHIBIT_APPLET_NAME);
 	gtk_about_dialog_set_version (about, VERSION);
-	gtk_about_dialog_set_copyright (about, _("Copyright \xc2\xa9 2006 Richard Hughes"));
+	gtk_about_dialog_set_copyright (about, _("Copyright \xc2\xa9 2006-2007 Richard Hughes"));
 	gtk_about_dialog_set_comments (about, GPM_INHIBIT_APPLET_DESC);
 	gtk_about_dialog_set_authors (about, authors);
 	gtk_about_dialog_set_documenters (about, documenters);
