@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2005-2007 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -29,6 +29,122 @@
 #include "gpm-debug.h"
 #include "gpm-common.h"
 
+/**
+ * gpm_precision_round_up:
+ * @value: The input value
+ * @smallest: The smallest increment allowed
+ *
+ * 101, 10	110
+ * 95,  10	100
+ * 0,   10	10
+ * 112, 10	120
+ * 100, 10	100
+ **/
+guint
+gpm_precision_round_up (guint value, guint smallest)
+{
+	guint division;
+	if (value == 0) {
+		return smallest;
+	}
+	if (smallest == 0) {
+		gpm_warning ("divisor zero");
+		return 0;
+	}
+	division = value / smallest;
+	/* add one unit to scale if we can't contain */
+	if (value % smallest != 0) {
+		division++;
+	}
+	division *= smallest;
+	return division;
+}
+
+/**
+ * gpm_precision_round_down:
+ * @value: The input value
+ * @smallest: The smallest increment allowed
+ *
+ * 101, 10	100
+ * 95,  10	90
+ * 0,   10	0
+ * 112, 10	110
+ * 100, 10	100
+ **/
+guint
+gpm_precision_round_down (guint value, guint smallest)
+{
+	guint division;
+	if (value == 0) {
+		return 0;
+	}
+	if (smallest == 0) {
+		gpm_warning ("divisor zero");
+		return 0;
+	}
+	division = value / smallest;
+	division *= smallest;
+	return division;
+}
+
+/**
+ * gpm_rgb_to_colour:
+ * @red: The red value
+ * @green: The green value
+ * @blue: The blue value
+ **/
+guint32
+gpm_rgb_to_colour (guint8 red, guint8 green, guint8 blue)
+{
+	guint32 colour = 0;
+	colour += (guint32) red * 0x10000;
+	colour += (guint32) green * 0x100;
+	colour += (guint32) blue;
+	return colour;
+}
+
+/**
+ * gpm_colour_to_rgb:
+ * @red: The red value
+ * @green: The green value
+ * @blue: The blue value
+ **/
+void
+gpm_colour_to_rgb (guint32 colour, guint8 *red, guint8 *green, guint8 *blue)
+{
+	*red = (colour & 0xff0000) / 0x10000;
+	*green = (colour & 0x00ff00) / 0x100;
+	*blue = colour & 0x0000ff;
+}
+
+/**
+ * gpm_exponential_average:
+ * @previous: The old value
+ * @new: The new value
+ * @slew: The slew rate as a percentage
+ *
+ * We should do an exponentially weighted average so that high frequency
+ * changes are smoothed. This should mean the output does not change
+ * drastically between updates.
+ **/
+gint
+gpm_exponential_average (gint previous, gint new, guint slew)
+{
+	gint result = 0;
+	gfloat factor = 0;
+	gfloat factor_inv = 1;
+	if (previous == 0 || slew == 0) {
+		/* startup, or re-initialization - we have no data */
+		gpm_debug ("Quoting output with only one value...");
+		result = new;
+	} else {
+		factor = (gfloat) slew / 100.0f;
+		factor_inv = 1.0f - factor;
+		result = (gint) ((factor_inv * (gfloat) new) + (factor * (gfloat) previous));
+	}
+	return result;
+}
+
 /**
  * gpm_percent_to_discrete:
  * @percentage: The percentage to convert
@@ -47,6 +163,10 @@ gpm_percent_to_discrete (guint percentage,
 	if (percentage > 100) {
 		return levels;
 	}
+	if (levels == 0) {
+		gpm_warning ("levels is 0!");
+		return 0;
+	}
 	return ((gfloat) percentage * (gfloat) (levels - 1)) / 100.0f;
 }
 
@@ -67,6 +187,10 @@ gpm_discrete_to_percent (guint discrete,
 	if (discrete > levels) {
 		return 100;
 	}
+	if (levels == 0) {
+		gpm_warning ("levels is 0!");
+		return 0;
+	}
 	return (guint) ((float) discrete * (100.0f / (float) (levels - 1)));
 }
 
@@ -112,7 +236,7 @@ gpm_get_timestring (guint time_secs)
 	else
 		/* TRANSLATOR: "%i %s %i %s" are "%i hours %i minutes"
 		 * Swap order with "%2$s %2$i %1$s %1$i if needed */
-		timestring = g_strdup_printf (_("%i %s, %i %s"),
+		timestring = g_strdup_printf (_("%i %s %i %s"),
 				hours, ngettext ("hour", "hours", hours),
 				minutes, ngettext ("minute", "minutes", minutes));
 	return timestring;
