@@ -21,30 +21,33 @@
 
 #include "config.h"
 #include <gtk/gtk.h>
+#include <pango/pangocairo.h>
 #include <glib/gi18n.h>
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "gpm-common.h"
 #include "gpm-graph-widget.h"
-#include "gpm-info-data.h"
+#include "gpm-array.h"
 #include "gpm-debug.h"
 
 G_DEFINE_TYPE (GpmGraphWidget, gpm_graph_widget, GTK_TYPE_DRAWING_AREA);
 #define GPM_GRAPH_WIDGET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_GRAPH_WIDGET, GpmGraphWidgetPrivate))
+#define GPM_GRAPH_WIDGET_FONT "Sans 8"
 
 struct GpmGraphWidgetPrivate
 {
 	gboolean		 use_grid;
 	gboolean		 use_legend;
-	gboolean		 use_axis_labels;
 	gboolean		 use_events;
 
 	gboolean		 invert_x;
 	gboolean		 autorange_x;
 	gboolean		 invert_y;
 
-	GSList			*keyvals; /* to hold all the key data */
+	GSList			*key_data; /* lines */
+	GSList			*key_event; /* dots */
 
 	gint			 stop_x;
 	gint			 stop_y;
@@ -60,25 +63,18 @@ struct GpmGraphWidgetPrivate
 
 	GpmGraphWidgetAxisType	 axis_type_x;
 	GpmGraphWidgetAxisType	 axis_type_y;
-	const gchar		*axis_label_x;
-	const gchar		*axis_label_y;
 	gchar			*title;
 
 	cairo_t			*cr;
-	cairo_font_options_t	*options;
+	PangoLayout 		*layout;
 
 	GPtrArray		*data_list;
-	GList			*events;
+	GpmArray		*events;
 };
 
 static gboolean gpm_graph_widget_expose (GtkWidget *graph, GdkEventExpose *event);
 static void	gpm_graph_widget_finalize (GObject *object);
 
-#define GPM_GRAPH_WIDGET_KEY_MAX	14
-#define GPM_CHARGED_TEXT		_("Charged")
-#define GPM_CHARGING_TEXT		_("Charging")
-#define GPM_DISCHARGING_TEXT		_("Discharging")
-
 /**
  * gpm_graph_widget_key_compare_func
  * Return value: 0 if cookie matches
@@ -103,7 +99,7 @@ gpm_graph_widget_key_find_id (GpmGraphWidget *graph, guint id)
 {
 	GpmGraphWidgetKeyItem *data;
 	GSList *ret;
-	ret = g_slist_find_custom (graph->priv->keyvals, &id,
+	ret = g_slist_find_custom (graph->priv->key_event, &id,
 				   gpm_graph_widget_key_compare_func);
 	if (! ret) {
 		return NULL;
@@ -113,31 +109,99 @@ gpm_graph_widget_key_find_id (GpmGraphWidget *graph, guint id)
 }
 
 /**
- * gpm_graph_widget_key_add:
+ * gpm_graph_widget_key_event_add:
  **/
 gboolean
-gpm_graph_widget_key_add (GpmGraphWidget       *graph,
-			  const gchar	       *name,
+gpm_graph_widget_key_event_add (GpmGraphWidget *graph,
 			  guint		        id,
-			  GpmGraphWidgetColour  colour,
-			  GpmGraphWidgetShape   shape)
+			  guint32               colour,
+			  GpmGraphWidgetShape   shape,
+			  const gchar	       *desc)
 {
 	GpmGraphWidgetKeyItem *keyitem;
 	g_return_val_if_fail (graph != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), FALSE);
 
-	gpm_debug ("add to hashtable %s", name);
+	gpm_debug ("add to hashtable '%s'", desc);
 	keyitem = g_new0 (GpmGraphWidgetKeyItem, 1);
 
 	keyitem->id = id;
-	keyitem->name = name;
+	keyitem->desc = g_strdup (desc);
 	keyitem->colour = colour;
 	keyitem->shape = shape;
 
-	graph->priv->keyvals = g_slist_append (graph->priv->keyvals, (gpointer) keyitem);
+	graph->priv->key_event = g_slist_append (graph->priv->key_event, (gpointer) keyitem);
 	return TRUE;
 }
 
+/**
+ * gpm_graph_widget_key_data_clear:
+ **/
+gboolean
+gpm_graph_widget_key_data_clear (GpmGraphWidget *graph)
+{
+	GpmGraphWidgetKeyData *keyitem;
+	guint a;
+
+	g_return_val_if_fail (graph != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), FALSE);
+
+	/* remove items in list and free */
+	for (a=0; a<g_slist_length (graph->priv->key_data); a++) {
+		keyitem = (GpmGraphWidgetKeyData *) g_slist_nth_data (graph->priv->key_data, a);
+		g_free (keyitem->desc);
+		g_free (keyitem);
+	}
+	g_slist_free (graph->priv->key_data);
+	graph->priv->key_data = NULL;
+
+	return TRUE;
+}
+
+/**
+ * gpm_graph_widget_key_event_clear:
+ **/
+gboolean
+gpm_graph_widget_key_event_clear (GpmGraphWidget *graph)
+{
+	GpmGraphWidgetKeyItem *keyitem;
+	guint a;
+
+	g_return_val_if_fail (graph != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), FALSE);
+
+	/* remove items in list and free */
+	for (a=0; a<g_slist_length (graph->priv->key_event); a++) {
+		keyitem = (GpmGraphWidgetKeyItem *) g_slist_nth_data (graph->priv->key_event, a);
+		g_free (keyitem->desc);
+		g_free (keyitem);
+	}
+	g_slist_free (graph->priv->key_event);
+	graph->priv->key_event = NULL;
+
+	return TRUE;
+}
+
+/**
+ * gpm_graph_widget_key_data_add:
+ **/
+gboolean
+gpm_graph_widget_key_data_add (GpmGraphWidget *graph, guint colour, const gchar *desc)
+{
+	GpmGraphWidgetKeyData *keyitem;
+
+	g_return_val_if_fail (graph != NULL, FALSE);
+	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), FALSE);
+
+	gpm_debug ("add to list %s", desc);
+	keyitem = g_new0 (GpmGraphWidgetKeyData, 1);
+
+	keyitem->colour = colour;
+	keyitem->desc = g_strdup (desc);
+
+	graph->priv->key_data = g_slist_append (graph->priv->key_data, (gpointer) keyitem);
+	return TRUE;
+}
 
 
 /**
@@ -151,7 +215,6 @@ GpmGraphWidgetAxisType
 gpm_graph_widget_string_to_axis_type (const gchar *type)
 {
 	GpmGraphWidgetAxisType ret;
-	g_return_val_if_fail (type != NULL, GPM_GRAPH_WIDGET_TYPE_INVALID);
 
 	ret = GPM_GRAPH_WIDGET_TYPE_INVALID;
 	if (strcmp (type, "percentage") == 0) {
@@ -163,56 +226,10 @@ gpm_graph_widget_string_to_axis_type (const gchar *type)
 	} else if (strcmp (type, "voltage") == 0) {
 		ret = GPM_GRAPH_WIDGET_TYPE_VOLTAGE;
 	}
-	
+
 	return ret;
 }
 
-/**
- * gpm_graph_widget_get_axis_label_y:
- * @graph: This class instance
- * @type: The axis type, e.g. GPM_GRAPH_WIDGET_TYPE_TIME
- **/
-static const gchar *
-gpm_graph_widget_get_axis_label_y (GpmGraphWidget *graph, GpmGraphWidgetAxisType type)
-{
-	g_return_val_if_fail (graph != NULL, NULL);
-	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), NULL);
-
-	if (type == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
-		return _("Charge percentage");
-	}
-	if (type == GPM_GRAPH_WIDGET_TYPE_TIME) {
-		return _("Time remaining");
-	}
-	if (type == GPM_GRAPH_WIDGET_TYPE_POWER) {
-		return _("Power");
-	}
-	if (type == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
-		return _("Cell Voltage");
-	}
-	return _("Unknown caption");
-}
-
-/**
- * gpm_graph_widget_get_axis_label_x:
- * @graph: This class instance
- * @type: The axis type, e.g. GPM_GRAPH_WIDGET_TYPE_TIME
- **/
-static const gchar *
-gpm_graph_widget_get_axis_label_x (GpmGraphWidget *graph, GpmGraphWidgetAxisType type)
-{
-	g_return_val_if_fail (graph != NULL, NULL);
-	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), NULL);
-
-	if (type == GPM_GRAPH_WIDGET_TYPE_TIME) {
-		/* I want this translated please */
-		const char *moo;
-		moo = _("Time");
-		return _("Time since startup");
-	}
-	return _("Unknown caption");
-}
-
 /**
  * gpm_graph_widget_set_title:
  * @graph: This class instance
@@ -240,7 +257,6 @@ gpm_graph_widget_set_axis_type_x (GpmGraphWidget *graph, GpmGraphWidgetAxisType
 	g_return_if_fail (graph != NULL);
 	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
 	graph->priv->axis_type_x = axis;
-	graph->priv->axis_label_x = gpm_graph_widget_get_axis_label_x (graph, axis);
 }
 
 /**
@@ -254,7 +270,6 @@ gpm_graph_widget_set_axis_type_y (GpmGraphWidget *graph, GpmGraphWidgetAxisType
 	g_return_if_fail (graph != NULL);
 	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
 	graph->priv->axis_type_y = axis;
-	graph->priv->axis_label_y = gpm_graph_widget_get_axis_label_y (graph, axis);
 }
 
 /**
@@ -273,22 +288,6 @@ gpm_graph_widget_enable_legend (GpmGraphWidget *graph, gboolean enable)
 	gtk_widget_show (GTK_WIDGET (graph));
 }
 
-/**
- * gpm_graph_widget_enable_axis_labels:
- * @graph: This class instance
- * @enable: If we should show the axis labels
- **/
-void
-gpm_graph_widget_enable_axis_labels (GpmGraphWidget *graph, gboolean enable)
-{
-	g_return_if_fail (graph != NULL);
-	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
-	graph->priv->use_axis_labels = enable;
-
-	gtk_widget_hide (GTK_WIDGET (graph));
-	gtk_widget_show (GTK_WIDGET (graph));
-}
-
 /**
  * gpm_graph_widget_enable_events:
  * @graph: This class instance
@@ -328,6 +327,10 @@ gpm_graph_widget_class_init (GpmGraphWidgetClass *class)
 static void
 gpm_graph_widget_init (GpmGraphWidget *graph)
 {
+	PangoFontMap *fontmap;
+	PangoContext *context;
+	PangoFontDescription *desc;
+
 	graph->priv = GPM_GRAPH_WIDGET_GET_PRIVATE (graph);
 	graph->priv->invert_x = FALSE;
 	graph->priv->invert_y = FALSE;
@@ -337,17 +340,25 @@ gpm_graph_widget_init (GpmGraphWidget *graph)
 	graph->priv->stop_y = 100;
 	graph->priv->use_grid = TRUE;
 	graph->priv->use_legend = FALSE;
-	graph->priv->use_axis_labels = FALSE;
 	graph->priv->autorange_x = TRUE;
 	graph->priv->data_list = g_ptr_array_new ();
-	graph->priv->keyvals = NULL;
-	graph->priv->axis_label_x = NULL;
-	graph->priv->axis_label_y = NULL;
+	graph->priv->key_data = NULL;
+	graph->priv->key_event = NULL;
 	graph->priv->title = NULL;
 	graph->priv->axis_type_x = GPM_GRAPH_WIDGET_TYPE_TIME;
 	graph->priv->axis_type_y = GPM_GRAPH_WIDGET_TYPE_PERCENTAGE;
-	/* setup font */
-	graph->priv->options = cairo_font_options_create ();
+
+	/* do pango stuff */
+	fontmap = pango_cairo_font_map_get_default ();
+	context = pango_cairo_font_map_create_context (PANGO_CAIRO_FONT_MAP (fontmap));
+
+	/* Comment out as this requires pango 1.16 when FC6 only supports 1.14
+	pango_context_set_base_gravity (context, PANGO_GRAVITY_AUTO); */
+
+	graph->priv->layout = pango_layout_new (context);
+	desc = pango_font_description_from_string (GPM_GRAPH_WIDGET_FONT);
+	pango_layout_set_font_description (graph->priv->layout, desc);
+	pango_font_description_free (desc);
 }
 
 /**
@@ -357,24 +368,20 @@ gpm_graph_widget_init (GpmGraphWidget *graph)
 static void
 gpm_graph_widget_finalize (GObject *object)
 {
-	guint a;
+	PangoContext *context;
 	GpmGraphWidget *graph = (GpmGraphWidget*) object;
-	cairo_font_options_destroy (graph->priv->options);
 
-	GpmGraphWidgetKeyItem *keyitem;
-	/* remove items in list and free */
-	for (a=0; a<g_slist_length (graph->priv->keyvals); a++) {
-		keyitem = (GpmGraphWidgetKeyItem *) g_slist_nth_data (graph->priv->keyvals, a);
-		g_free (keyitem);
-	}
-	g_slist_free (graph->priv->keyvals);
+	/* clear key */
+	gpm_graph_widget_key_data_clear (graph);
+	gpm_graph_widget_key_event_clear (graph);
+
 	g_ptr_array_free (graph->priv->data_list, FALSE);
 
-	g_free (graph->priv->title);
+	context = pango_layout_get_context (graph->priv->layout);
+	g_object_unref (graph->priv->layout);
+	g_object_unref (context);
 
-	if (graph->priv->events) {
-		g_list_free (graph->priv->events);
-	}
+	g_free (graph->priv->title);
 }
 
 /**
@@ -415,19 +422,25 @@ gpm_graph_widget_set_invert_y (GpmGraphWidget *graph, gboolean inv)
  * Sets the data for the graph. You MUST NOT free the list before the widget.
  **/
 void
-gpm_graph_widget_set_data (GpmGraphWidget *graph, GList *list, guint id)
+gpm_graph_widget_set_data (GpmGraphWidget *graph, GpmArray *array, guint id)
 {
+	g_return_if_fail (array != NULL);
 	g_return_if_fail (graph != NULL);
 	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
 
+	if (gpm_array_get_size (array) == 0) {
+		gpm_warning ("Trying to assign a zero length array");
+		return;
+	}
+
 	/* fresh list */
 	if (graph->priv->data_list->len == 0 || graph->priv->data_list->len < id + 1) {
-		g_ptr_array_add (graph->priv->data_list, (gpointer) list);
+		g_ptr_array_add (graph->priv->data_list, (gpointer) array);
 	} else {
 		/* remove existing, and add new */
 		gpm_debug ("Re-assigning dataset");
 		g_ptr_array_remove_index (graph->priv->data_list, id);
-		g_ptr_array_add (graph->priv->data_list, (gpointer) list);
+		g_ptr_array_add (graph->priv->data_list, (gpointer) array);
 	}
 
 }
@@ -440,15 +453,12 @@ gpm_graph_widget_set_data (GpmGraphWidget *graph, GList *list, guint id)
  * Sets the data for the graph. You MUST NOT free the list before the widget.
  **/
 void
-gpm_graph_widget_set_events (GpmGraphWidget *graph, GList *list)
+gpm_graph_widget_set_events (GpmGraphWidget *graph, GpmArray *array)
 {
 	g_return_if_fail (graph != NULL);
 	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
 
-	if (graph->priv->events) {
-		g_list_free (graph->priv->events);
-	}
-	graph->priv->events = list;
+	graph->priv->events = array;
 }
 
 /**
@@ -564,16 +574,12 @@ gpm_graph_widget_draw_labels (GpmGraphWidget *graph, cairo_t *cr)
 	gfloat divheight = (gfloat)graph->priv->box_height / 10.0f;
 	gint length_x = graph->priv->stop_x - graph->priv->start_x;
 	gint length_y = graph->priv->stop_y - graph->priv->start_y;
-	cairo_text_extents_t extents;
+	PangoRectangle ink_rect, logical_rect;
 	gfloat offsetx = 0;
 	gfloat offsety = 0;
 
 	cairo_save (cr);
 
-	cairo_set_font_options (cr, graph->priv->options);
-	cairo_select_font_face(cr, "Sans", CAIRO_FONT_SLANT_NORMAL,
-					CAIRO_FONT_WEIGHT_NORMAL);
-
 	/* do x text */
 	cairo_set_source_rgb (cr, 0, 0, 0);
 	for (a=0; a<11; a++) {
@@ -585,20 +591,21 @@ gpm_graph_widget_draw_labels (GpmGraphWidget *graph, cairo_t *cr)
 		}
 		text = gpm_get_axis_label (graph->priv->axis_type_x, value);
 
-		cairo_text_extents (cr, text, &extents);
+		pango_layout_set_text (graph->priv->layout, text, -1);
+		pango_layout_get_pixel_extents (graph->priv->layout, &ink_rect, &logical_rect);
 		/* have data points 0 and 10 bounded, but 1..9 centered */
 		if (a == 0) {
-			offsetx = 2;
+			offsetx = 2.0;
 		} else if (a == 10) {
-			offsetx = extents.width;
+			offsetx = ink_rect.width;
 		} else {
-			offsetx = (extents.width / 2.0f);
+			offsetx = (ink_rect.width / 2.0f);
 		}
 
 		cairo_move_to (cr, b - offsetx,
-			       graph->priv->box_y + graph->priv->box_height + 15);
+			       graph->priv->box_y + graph->priv->box_height + 2.0);
 
-		cairo_show_text (cr, text);
+		pango_cairo_show_layout (cr, graph->priv->layout);
 		g_free (text);
 	}
 
@@ -612,18 +619,21 @@ gpm_graph_widget_draw_labels (GpmGraphWidget *graph, cairo_t *cr)
 		}
 		text = gpm_get_axis_label (graph->priv->axis_type_y, value);
 
-		cairo_text_extents (cr, text, &extents);
+		pango_layout_set_text (graph->priv->layout, text, -1);
+		pango_layout_get_pixel_extents (graph->priv->layout, &ink_rect, &logical_rect);
+
 		/* have data points 0 and 10 bounded, but 1..9 centered */
 		if (a == 10) {
 			offsety = 0;
 		} else if (a == 0) {
-			offsety = extents.height;
+			offsety = ink_rect.height;
 		} else {
-			offsety = (extents.height / 2.0f);
+			offsety = (ink_rect.height / 2.0f);
 		}
-		offsetx = extents.width + 5;
-		cairo_move_to (cr, graph->priv->box_x - offsetx, b + offsety);
-		cairo_show_text (cr, text);
+		offsetx = ink_rect.width + 7;
+		offsety -= 10;
+		cairo_move_to (cr, graph->priv->box_x - offsetx - 2, b + offsety);
+		pango_cairo_show_layout (cr, graph->priv->layout);
 		g_free (text);
 	}
 
@@ -645,10 +655,11 @@ gpm_graph_widget_auto_range (GpmGraphWidget *graph)
 	gint biggest_y = 0;
 	gint smallest_x = 999999;
 	gint smallest_y = 999999;
-	GpmInfoDataPoint *new = NULL;
-	GList *l;
-	GList *list;
+	GpmArray *array;
+	GpmArrayPoint *point;
 	guint i;
+	guint j;
+	guint length;
 
 	if (graph->priv->data_list->len == 0) {
 		gpm_debug ("no data");
@@ -661,33 +672,30 @@ gpm_graph_widget_auto_range (GpmGraphWidget *graph)
 
 	/* get the range for all graphs */
 	for (i=0; i<graph->priv->data_list->len; i++) {
-		list = (GList *) g_ptr_array_index (graph->priv->data_list, i);
-		for (l=list; l != NULL; l=l->next) {
-			new = (GpmInfoDataPoint *) l->data;
-			if (new->time > biggest_x) {
-				biggest_x = new->time;
+		array = g_ptr_array_index (graph->priv->data_list, i);
+		length = gpm_array_get_size (array);
+		for (j=0; j < length; j++) {
+			point = gpm_array_get (array, j);
+			if (point->x > biggest_x) {
+				biggest_x = point->x;
 			}
-			if (new->value > biggest_y) {
-				biggest_y = new->value;
+			if (point->y > biggest_y) {
+				biggest_y = point->y;
 			}
-			if (new->time < smallest_x) {
-				smallest_x = new->time;
+			if (point->x < smallest_x) {
+				smallest_x = point->x;
 			}
-			if (new->value < smallest_y) {
-				smallest_y = new->value;
+			if (point->y < smallest_y) {
+				smallest_y = point->y;
 			}
-		}		
+		}
 	}
 	gpm_debug ("Data range is %i<x<%i, %i<y<%i", smallest_x, biggest_x, smallest_y, biggest_y);
 
 	/* do we autorange the start (so it starts at non-zero)? */
 	if (graph->priv->autorange_x) {
 		/* x is always time and always autoranges to the minute scale */
-		smallest_x = (smallest_x / 60) * 60;
-		if (smallest_x < 60) {
-			smallest_x = 0;
-		}
-		graph->priv->start_x = smallest_x;
+		graph->priv->start_x = gpm_precision_round_down (smallest_x, 10*60);
 	} else {
 		graph->priv->start_x = 0;
 	}
@@ -696,19 +704,17 @@ gpm_graph_widget_auto_range (GpmGraphWidget *graph)
 	if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
 		graph->priv->stop_x = 100;
 	} else if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_TIME) {
-		graph->priv->stop_x = (((biggest_x - smallest_x) / (10 * 60)) + 1) * (10 * 60) + smallest_x;
+		if (biggest_x > 10*60*60) {
+			graph->priv->stop_x = gpm_precision_round_up ((biggest_x - smallest_x), 10*60*60) + smallest_x;
+		} else {
+			graph->priv->stop_x = gpm_precision_round_up ((biggest_x - smallest_x), 10*60) + smallest_x;
+		}
 	} else if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_POWER) {
-		graph->priv->stop_x = (((biggest_x - smallest_x) / 10000) + 2) * 10000 + smallest_x;
-		if (graph->priv->stop_x < 10000) {
-			graph->priv->stop_x = 10000 + smallest_x;
-		}
+		graph->priv->stop_x = gpm_precision_round_up ((biggest_x - smallest_x), 10000) + smallest_x;
 	} else if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
-		graph->priv->stop_x = (((biggest_x - smallest_x) / 1000) + 2) * 1000 + smallest_x;
-		if (graph->priv->stop_x < 1000) {
-			graph->priv->stop_x = 1000 + smallest_x;
-		}
+		graph->priv->stop_x = gpm_precision_round_up ((biggest_x - smallest_x), 1000) + smallest_x;
 	} else {
-		graph->priv->start_x = smallest_x;
+		graph->priv->start_x = 0;
 		graph->priv->stop_x = biggest_x;
 	}
 
@@ -718,27 +724,19 @@ gpm_graph_widget_auto_range (GpmGraphWidget *graph)
 		graph->priv->stop_y = 100;
 	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_TIME) {
 		graph->priv->start_y = 0;
-		graph->priv->stop_y = ((biggest_y / 60) + 2)* 60;
-		if (graph->priv->stop_y > 60) {
-			graph->priv->stop_y = ((biggest_y / (10 * 60)) + 2) * (10 * 60);
-		}
-		if (graph->priv->stop_y < 60) {
-			graph->priv->stop_y = 60;
+		if (biggest_y > 5*60*60) {
+			graph->priv->stop_y = gpm_precision_round_up (biggest_y, 5*60*60);
+		} else {
+			graph->priv->stop_y = gpm_precision_round_up (biggest_y, 5*60);
 		}
 	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_POWER) {
 		graph->priv->start_y = 0;
-		graph->priv->stop_y = ((biggest_y / 10000) + 2) * 10000;
-		if (graph->priv->stop_y < 10000) {
-			graph->priv->stop_y = 10000;
-		}
+		graph->priv->stop_y = gpm_precision_round_up (biggest_y, 10000);
 	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
 		graph->priv->start_y = 0;
-		graph->priv->stop_y = ((biggest_y / 1000) + 2) * 1000;
-		if (graph->priv->stop_y < 1000) {
-			graph->priv->stop_y = 1000;
-		}
+		graph->priv->stop_y = gpm_precision_round_up (biggest_y, 1000);
 	} else {
-		graph->priv->start_y = smallest_y;
+		graph->priv->start_y = 0;
 		graph->priv->stop_y = biggest_y;
 	}
 	gpm_debug ("Processed range is %i<x<%i, %i<y<%i",
@@ -752,45 +750,11 @@ gpm_graph_widget_auto_range (GpmGraphWidget *graph)
  * @colour: The colour enum
  **/
 static void
-gpm_graph_widget_set_colour (cairo_t *cr, GpmGraphWidgetColour colour)
+gpm_graph_widget_set_colour (cairo_t *cr, guint32 colour)
 {
-	if (colour == GPM_GRAPH_WIDGET_COLOUR_DEFAULT) {
-		cairo_set_source_rgb (cr, 0, 0.7, 0);
-	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_BLACK) {
-		cairo_set_source_rgb (cr, 0, 0, 0);
-	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_WHITE) {
-		cairo_set_source_rgb (cr, 1, 1, 1);
-	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_RED) {
-		cairo_set_source_rgb (cr, 1, 0, 0);
-	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_BLUE) {
-		cairo_set_source_rgb (cr, 0, 0, 1);
-	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_GREEN) {
-		cairo_set_source_rgb (cr, 0, 1, 0);
-	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_MAGENTA) {
-		cairo_set_source_rgb (cr, 1, 0, 1);
-	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_YELLOW) {
-		cairo_set_source_rgb (cr, 1, 1, 0);
-	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_CYAN) {
-		cairo_set_source_rgb (cr, 0, 1, 1);
-	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_GREY) {
-		cairo_set_source_rgb (cr, 0.8, 0.8, 0.8);
-	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_DARK_RED) {
-		cairo_set_source_rgb (cr, 0.5, 0, 0);
-	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_DARK_BLUE) {
-		cairo_set_source_rgb (cr, 0, 0, 0.5);
-	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_DARK_GREEN) {
-		cairo_set_source_rgb (cr, 0, 0.5, 0);
-	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_DARK_MAGENTA) {
-		cairo_set_source_rgb (cr, 0.5, 0, 0.5);
-	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_DARK_YELLOW) {
-		cairo_set_source_rgb (cr, 0.5, 0.5, 0);
-	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_DARK_CYAN) {
-		cairo_set_source_rgb (cr, 0, 0.5, 0.5);
-	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_DARK_GREY) {
-		cairo_set_source_rgb (cr, 0.5, 0.5, 0.5);
-	} else {
-		gpm_warning ("Unknown colour: %i", colour);
-	}
+	guint8 r, g, b;
+	gpm_colour_to_rgb (colour, &r, &g, &b);
+	cairo_set_source_rgb (cr, ((gdouble) r)/256.0f, ((gdouble) g)/256.0f, ((gdouble) b)/256.0f);
 }
 
 /**
@@ -807,7 +771,7 @@ static void
 gpm_graph_widget_draw_dot (cairo_t             *cr,
 			   gfloat               x,
 			   gfloat               y,
-			   GpmGraphWidgetColour colour,
+			   guint32              colour,
 			   GpmGraphWidgetShape  shape)
 {
 	gfloat width;
@@ -883,7 +847,7 @@ gpm_graph_widget_draw_dot (cairo_t             *cr,
  * Draw the legend line on the graph of a specified colour
  **/
 static void
-gpm_graph_widget_draw_legend_line (cairo_t *cr, gfloat x, gfloat y, GpmGraphWidgetColour colour)
+gpm_graph_widget_draw_legend_line (cairo_t *cr, gfloat x, gfloat y, guint32 colour)
 {
 	gfloat width = 10;
 	gfloat height = 2;
@@ -913,55 +877,6 @@ gpm_graph_widget_get_pos_on_graph (GpmGraphWidget *graph, gfloat data_x, gfloat
 	*y = graph->priv->box_y + (graph->priv->unit_y * (gfloat)(graph->priv->stop_y - data_y)) + 1.5;
 }
 
-/**
- * gpm_graph_widget_interpolate_value:
- * @this: The first data point
- * @last: The other data point
- * @xintersect: The x value (i.e. the x we provide)
- * Return value: The interpolated value, or 0 if invalid
- *
- * Interpolates onto the graph in the y direction. If only supplied one point
- * then don't interpolate.
- **/
-static gint
-gpm_graph_widget_interpolate_value (GpmInfoDataPoint *this,
-			     GpmInfoDataPoint *last,
-			     gint xintersect)
-{
-	gint dy, dx;
-	gfloat m;
-	gint c;
-	gint y;
-
-	/* we have no points */
-	if (! this) {
-		return 0;
-	}
-
-	/* we only have one point, don't interpolate */
-	if (! last) {
-		return this->value;
-	}
-
-	/* gradient */
-	dx = this->time - last->time;
-	dy = this->value - last->value;
-	m = (gfloat) dy / (gfloat) dx;
-
-	/* y-intersect */
-	c = (-m * (gfloat) this->time) + this->value;
-
-	/* y = mx + c */
-	y = (m * (gfloat) xintersect) + c;
-
-	/* limit the y intersect to the last height, so we don't extend the
-	 * graph into the unknown */
-	if (y > this->value) {
-		y = this->value;
-	}
-	return y;
-}
-
 /**
  * gpm_graph_widget_draw_line:
  * @graph: This class instance
@@ -975,10 +890,10 @@ gpm_graph_widget_draw_line (GpmGraphWidget *graph, cairo_t *cr)
 {
 	gfloat oldx, oldy;
 	gfloat newx, newy;
-	GpmInfoDataPoint *eventdata;
-	GpmGraphWidgetKeyItem *keyitem;
-	GList *l;
-	GpmInfoDataPoint *new;
+	guint j;
+	guint length;
+	GpmArray *array;
+	GpmArrayPoint *point;
 	guint i;
 
 	if (graph->priv->data_list->len == 0) {
@@ -987,30 +902,32 @@ gpm_graph_widget_draw_line (GpmGraphWidget *graph, cairo_t *cr)
 	}
 	cairo_save (cr);
 
-	/* do the lines on the graphs */
+	/* do all the lines on the graphs */
 	for (i=0; i<graph->priv->data_list->len; i++) {
 		gpm_debug ("drawing line %i", i);
-		l = (GList *) g_ptr_array_index (graph->priv->data_list, i);
-		if (l == NULL) {
-			gpm_warning ("list item NULL!");
+		array = g_ptr_array_index (graph->priv->data_list, i);
+
+		/* we have no data */
+		if (array == NULL) {
 			break;
 		}
-		new = (GpmInfoDataPoint *) l->data;
+
+		/* get the very first point so we can work out the old */
+		point = gpm_array_get (array, 0);
 		oldx = 0;
 		oldy = 0;
-		gpm_graph_widget_get_pos_on_graph (graph, new->time, new->value, &oldx, &oldy);
-		for (l=l->next; l != NULL; l=l->next) {
-			new = (GpmInfoDataPoint *) l->data;
-			if (new == NULL) {
-				gpm_warning ("new item NULL!");
-				break;
-			}
-			/* do line */
-			gpm_graph_widget_get_pos_on_graph (graph, new->time, new->value, &newx, &newy);
+		gpm_graph_widget_get_pos_on_graph (graph, point->x, point->y, &oldx, &oldy);
+
+		length = gpm_array_get_size (array);
+		for (j=1; j < length; j++) {
+			point = gpm_array_get (array, j);
+
+			/* draw line */
+			gpm_graph_widget_get_pos_on_graph (graph, point->x, point->y, &newx, &newy);
 			cairo_move_to (cr, oldx, oldy);
 			cairo_line_to (cr, newx, newy);
 			cairo_set_line_width (cr, 1.5);
-			gpm_graph_widget_set_colour (cr, new->colour);
+			gpm_graph_widget_set_colour (cr, point->data);
 			cairo_stroke (cr);
 			/* save old */
 			oldx = newx;
@@ -1018,58 +935,78 @@ gpm_graph_widget_draw_line (GpmGraphWidget *graph, cairo_t *cr)
 		}
 	}
 
-	/* only do the events sometimes */
-	if (graph->priv->use_events) {
-		gint previous_point = 0;
-		gint prevpos = -1;
-		GpmInfoDataPoint *point_this = NULL;
-		GpmInfoDataPoint *point_last = NULL;
+	cairo_restore (cr);
+}
 
-		/* we track the list so we can put the point on the line */
-		l = (GList *) g_ptr_array_index (graph->priv->data_list, 0);
-		GList *l2 = l;
-		if (l2) {
-			point_this = (GpmInfoDataPoint *) l2->data;
-			l2 = l2->next;
+/**
+ * gpm_graph_widget_draw_event_dots:
+ * @graph: This class instance
+ * @cr: Cairo drawing context
+ *
+ * Draw the data line onto the graph with a big green line. We should already
+ * limit the data to < ~100 values, so this shouldn't take too long.
+ **/
+static void
+gpm_graph_widget_draw_event_dots (GpmGraphWidget *graph, cairo_t *cr)
+{
+	gfloat newx, newy;
+	GpmGraphWidgetKeyItem *keyitem;
+	guint i;
+	guint length;
+	GpmArray *array = NULL;
+	GpmArrayPoint *point;
+	gint dot;
+	guint previous_point = 0;
+	gint prevpos = -1;
+
+
+	if (graph->priv->events == NULL) {
+		/* we have no events */
+		return;
+	}
+
+	cairo_save (cr);
+
+	length = gpm_array_get_size (graph->priv->events);
+
+	/* always use the first data array */
+	if (graph->priv->data_list->len > 0) {
+		array = g_ptr_array_index (graph->priv->data_list, 0);
+	}
+
+	for (i=0; i < length; i++) {
+		point = gpm_array_get (graph->priv->events, i);
+		if (point == NULL) {
+			/* this shouldn't ever happen */
+			gpm_warning ("point NULL!");
+			break;
 		}
-		for (l=graph->priv->events; l != NULL; l=l->next) {
-			gint pos;
+		/* try to position the point on the line, or at zero if there is no line */
+		if (array == NULL) {
+			dot = 0;
+		} else {
+			dot = gpm_array_interpolate (array, point->x);
+		}
+		gpm_graph_widget_get_pos_on_graph (graph, point->x, dot, &newx, &newy);
 
-			eventdata = (GpmInfoDataPoint *) l->data;
-			if (eventdata == NULL) {
-				gpm_warning ("eventdata NULL!");
-				break;
-			}
-			/* If we have valid list data, go through the list data
-			   until we get a data point time value greater than the
-			   event we have. */
-			while (l2 && eventdata->time > point_this->time) {
-				point_last = point_this;
-				point_this = (GpmInfoDataPoint *) l2->data;
-				l2 = l2->next;
-			}
-			/* Interpolate in the y direction with the two
-			   previous points. This should be 100% accurate */
-			pos = gpm_graph_widget_interpolate_value (point_this,
-							       point_last,
-							       eventdata->time);
-			gpm_graph_widget_get_pos_on_graph (graph, eventdata->time,
-						    pos, &newx, &newy);
-			/* don't overlay the points, stack vertically */
-			if (abs (prevpos - newx) < 8) {
-				previous_point++;
+		/* don't overlay the points, stack vertically */
+		if (abs (newx - prevpos) < 10) {
+			previous_point++;
+		} else {
+			previous_point = 0;
+		}
+		newy -= (8 * previous_point);
+
+		/* only do the event dot, if it's going to be valid on the graph */
+		if (point->x > graph->priv->start_x && newy > graph->priv->box_y) {
+			keyitem = gpm_graph_widget_key_find_id (graph, point->y);
+			if (keyitem == NULL) {
+				gpm_warning ("did not find id %i", point->y);
 			} else {
-				previous_point = 0;
+				gpm_graph_widget_draw_dot (cr, newx, newy, keyitem->colour, keyitem->shape);
 			}
-			newy -= (8 * previous_point);
-			/* only do the event dot, if it's going to fit on the graph */
-			if (eventdata->time > graph->priv->start_x) {
-				keyitem = gpm_graph_widget_key_find_id (graph, eventdata->value);
-				gpm_graph_widget_draw_dot (cr, newx, newy,
-							   keyitem->colour, keyitem->shape);
-			}
-			prevpos = newx;
 		}
+		prevpos = newx;
 	}
 
 	cairo_restore (cr);
@@ -1101,25 +1038,6 @@ gpm_graph_widget_draw_bounding_box (cairo_t *cr,
 	cairo_stroke (cr);
 }
 
-/**
- * gpm_graph_widget_have_key_id:
- * Finds out if we have an event of type id.
- **/
-static gboolean
-gpm_graph_widget_have_key_id (GpmGraphWidget *graph, guint id)
-{
-	GpmInfoDataPoint *eventdata;
-	GList *l;
-	for (l=graph->priv->events; l != NULL; l=l->next) {
-		eventdata = (GpmInfoDataPoint *) l->data;
-		if (eventdata->value == id) {
-			return TRUE;
-		}
-	}
-	return FALSE;
-}
-
-
 /**
  * gpm_graph_widget_draw_legend:
  * @cr: Cairo drawing context
@@ -1138,43 +1056,43 @@ gpm_graph_widget_draw_legend (GpmGraphWidget *graph,
 	cairo_t *cr = graph->priv->cr;
 	gint y_count;
 	gint a;
-	GpmGraphWidgetKeyItem *keyitem;
+	GpmGraphWidgetKeyData *keydataitem;
+	GpmGraphWidgetKeyItem *keyeventitem;
 
 	gpm_graph_widget_draw_bounding_box (cr, x, y, width, height);
 	y_count = y + 10;
-	for (a=0; a<GPM_GRAPH_WIDGET_KEY_MAX; a++) {
-		keyitem = gpm_graph_widget_key_find_id (graph, a);
-		if (keyitem == NULL) {
-			break;
-		}
-		/* only do the legend point if we have one of these dots */
-		if (gpm_graph_widget_have_key_id (graph, keyitem->id) == TRUE) {
-			gpm_graph_widget_draw_dot (cr, x + 8, y_count,
-						   keyitem->colour, keyitem->shape);
-			cairo_move_to (cr, x + 8 + 10, y_count + 3);
-			cairo_show_text (cr, keyitem->name);
-			y_count = y_count + GPM_GRAPH_WIDGET_LEGEND_SPACING;
-		}
-	}
 
 	/* add the line colours to the legend */
-	gpm_graph_widget_draw_legend_line (cr, x + 8, y_count,
-					   GPM_GRAPH_WIDGET_COLOUR_CHARGING);
-	cairo_move_to (cr, x + 8 + 10, y_count + 3);
-	cairo_set_source_rgb (cr, 0, 0, 0);
-	cairo_show_text (cr, GPM_CHARGING_TEXT);
-	y_count = y_count + GPM_GRAPH_WIDGET_LEGEND_SPACING;
-	gpm_graph_widget_draw_legend_line (cr, x + 8, y_count,
-					   GPM_GRAPH_WIDGET_COLOUR_DISCHARGING);
-	cairo_move_to (cr, x + 8 + 10, y_count + 3);
-	cairo_set_source_rgb (cr, 0, 0, 0);
-	cairo_show_text (cr, GPM_DISCHARGING_TEXT);
-	y_count = y_count + GPM_GRAPH_WIDGET_LEGEND_SPACING;
-	gpm_graph_widget_draw_legend_line (cr, x + 8, y_count,
-					   GPM_GRAPH_WIDGET_COLOUR_CHARGED);
-	cairo_move_to (cr, x + 8 + 10, y_count + 3);
-	cairo_set_source_rgb (cr, 0, 0, 0);
-	cairo_show_text (cr, GPM_CHARGED_TEXT);
+	for (a=0; a<g_slist_length (graph->priv->key_data); a++) {
+		keydataitem = (GpmGraphWidgetKeyData *) g_slist_nth_data (graph->priv->key_data, a);
+		if (keydataitem == NULL) {
+			/* this shouldn't ever happen */
+			gpm_warning ("keydataitem NULL!");
+			break;
+		}
+		gpm_graph_widget_draw_legend_line (cr, x + 8, y_count, keydataitem->colour);
+		cairo_move_to (cr, x + 8 + 10, y_count - 6);
+		cairo_set_source_rgb (cr, 0, 0, 0);
+		pango_layout_set_text (graph->priv->layout, keydataitem->desc, -1);
+		pango_cairo_show_layout (cr, graph->priv->layout);
+		y_count = y_count + GPM_GRAPH_WIDGET_LEGEND_SPACING;
+	}
+
+	/* add the events to the legend */
+	for (a=0; a<g_slist_length (graph->priv->key_event); a++) {
+		keyeventitem = (GpmGraphWidgetKeyItem *) g_slist_nth_data (graph->priv->key_event, a);
+		if (keyeventitem == NULL) {
+			/* this shouldn't ever happen */
+			gpm_warning ("keyeventitem NULL!");
+			break;
+		}
+		gpm_graph_widget_draw_dot (cr, x + 8, y_count,
+					   keyeventitem->colour, keyeventitem->shape);
+		cairo_move_to (cr, x + 8 + 10, y_count - 6);
+		pango_layout_set_text (graph->priv->layout, keyeventitem->desc, -1);
+		pango_cairo_show_layout (cr, graph->priv->layout);
+		y_count = y_count + GPM_GRAPH_WIDGET_LEGEND_SPACING;
+	}
 }
 
 /**
@@ -1191,53 +1109,43 @@ static gboolean
 gpm_graph_widget_legend_calculate_size (GpmGraphWidget *graph, cairo_t *cr,
 					guint *width, guint *height)
 {
+	guint a;
+	PangoRectangle ink_rect, logical_rect;
+	GpmGraphWidgetKeyData *keydataitem;
+	GpmGraphWidgetKeyItem *keyeventitem;
+
 	g_return_val_if_fail (graph != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), FALSE);
 
-	cairo_set_font_options (cr, graph->priv->options);
-	cairo_select_font_face(cr, "Sans", CAIRO_FONT_SLANT_NORMAL,
-					CAIRO_FONT_WEIGHT_NORMAL);
-
-	guint a;
-	GpmGraphWidgetKeyItem *keyitem;
-	cairo_text_extents_t extents;
-
 	/* set defaults */
 	*width = 0;
-	*height = 3 * GPM_GRAPH_WIDGET_LEGEND_SPACING;
+	*height = 0;
 
-	/* get the max size for the labels (may be different for non UK) */
-	cairo_text_extents (cr, GPM_CHARGED_TEXT, &extents);
-	if (*width < extents.width) {
-		*width = extents.width;
-	}
-	cairo_text_extents (cr, GPM_CHARGING_TEXT, &extents);
-	if (*width < extents.width) {
-		*width = extents.width;
-	}
-	cairo_text_extents (cr, GPM_DISCHARGING_TEXT, &extents);
-	if (*width < extents.width) {
-		*width = extents.width;
-	}
-
-	/* find the width of the biggest key that is showing */
-	for (a=0; a<GPM_GRAPH_WIDGET_KEY_MAX; a++) {
-		keyitem = gpm_graph_widget_key_find_id (graph, a);
-		if (keyitem == NULL) {
-			break;
+	/* add the line colours to the legend */
+	for (a=0; a<g_slist_length (graph->priv->key_data); a++) {
+		keydataitem = (GpmGraphWidgetKeyData *) g_slist_nth_data (graph->priv->key_data, a);
+		*height = *height + GPM_GRAPH_WIDGET_LEGEND_SPACING;
+		pango_layout_set_text (graph->priv->layout, keydataitem->desc, -1);
+		pango_layout_get_pixel_extents (graph->priv->layout, &ink_rect, &logical_rect);
+		if (*width < ink_rect.width) {
+			*width = ink_rect.width;
 		}
-		/* only do the legend point if we have one of these dots */
-		if (gpm_graph_widget_have_key_id (graph, keyitem->id) == TRUE) {
-			*height = *height + GPM_GRAPH_WIDGET_LEGEND_SPACING + 2;
-			cairo_text_extents (cr, keyitem->name, &extents);
-			if (*width < extents.width) {
-				*width = extents.width;
-			}
+	}
+
+	/* add the events to the legend */
+	for (a=0; a<g_slist_length (graph->priv->key_event); a++) {
+		keyeventitem = (GpmGraphWidgetKeyItem *) g_slist_nth_data (graph->priv->key_event, a);
+		*height = *height + GPM_GRAPH_WIDGET_LEGEND_SPACING;
+		pango_layout_set_text (graph->priv->layout, keyeventitem->desc, -1);
+		pango_layout_get_pixel_extents (graph->priv->layout, &ink_rect, &logical_rect);
+		if (*width < ink_rect.width) {
+			*width = ink_rect.width;
 		}
 	}
 
 	/* add for borders */
 	*width += 25;
+	*height += 3;
 
 	return TRUE;
 }
@@ -1258,9 +1166,8 @@ gpm_graph_widget_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 	guint legend_width = 0;
 	gint data_x;
 	gint data_y;
-	cairo_text_extents_t extents_axis_label_x;
-	cairo_text_extents_t extents_axis_label_y;
-	cairo_text_extents_t extents_title;
+	PangoRectangle ink_rect_title;
+	PangoRectangle logical_rect;
 
 	GpmGraphWidget *graph = (GpmGraphWidget*) graph_widget;
 	g_return_if_fail (graph != NULL);
@@ -1270,32 +1177,17 @@ gpm_graph_widget_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 
 	cairo_save (cr);
 
-	cairo_select_font_face(cr, "Sans", CAIRO_FONT_SLANT_NORMAL,
-					CAIRO_FONT_WEIGHT_NORMAL);
-
 	graph->priv->box_x = 35;
 	graph->priv->box_y = 5;
 
 	if (graph->priv->title != NULL) {
-		cairo_text_extents (cr, graph->priv->title, &extents_title);
-		graph->priv->box_y += extents_title.height;
+		pango_layout_set_text (graph->priv->layout, graph->priv->title, -1);
+		pango_layout_get_pixel_extents (graph->priv->layout, &ink_rect_title, &logical_rect);
+		graph->priv->box_y += ink_rect_title.height;
 	}
 
 	graph->priv->box_height = graph_widget->allocation.height - (20 + graph->priv->box_y);
 
-	/* make size adjustment for axis labels */
-	if (graph->priv->use_axis_labels) {
-		/* get the size of the x axis text */
-		cairo_save (cr);
-		cairo_set_font_size (cr, 12);
-		cairo_text_extents (cr, graph->priv->axis_label_x, &extents_axis_label_x);
-		cairo_text_extents (cr, graph->priv->axis_label_y, &extents_axis_label_y);
-		cairo_restore (cr);
-
-		graph->priv->box_height -= (extents_axis_label_x.height + 2);
-		graph->priv->box_x += (extents_axis_label_y.height + 2);
-	}
-
 	/* make size adjustment for legend */
 	if (graph->priv->use_legend) {
 		graph->priv->box_width = graph_widget->allocation.width -
@@ -1309,8 +1201,9 @@ gpm_graph_widget_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 
 	/* center title */
 	if (graph->priv->title != NULL) {
-		cairo_move_to (cr, graph->priv->box_x + (graph->priv->box_width-extents_title.width)/2, 9);
-		cairo_show_text (cr, graph->priv->title);
+		cairo_move_to (cr, graph->priv->box_x + (graph->priv->box_width-ink_rect_title.width)/2, 9);
+		pango_layout_set_text(graph->priv->layout, graph->priv->title, -1);
+		pango_cairo_show_layout(cr, graph->priv->layout);
 	}
 
 	/* graph background */
@@ -1330,22 +1223,12 @@ gpm_graph_widget_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 	graph->priv->unit_y = (float)(graph->priv->box_height - 3) / (float) data_y;
 
 	gpm_graph_widget_draw_labels (graph, cr);
-
-	if (graph->priv->use_axis_labels) {
-		/* horizontal axis label */
-		cairo_save (cr);
-		cairo_set_font_size (cr, 12);
-		cairo_move_to (cr, graph->priv->box_x + ((graph->priv->box_width - extents_axis_label_x.width)/2), graph_widget->allocation.height - 3);
-		cairo_show_text (cr, graph->priv->axis_label_x);
-		/* vertical axis label */
-		cairo_move_to (cr, extents_axis_label_y.height, ((graph->priv->box_height + extents_axis_label_y.width) / 2) - graph->priv->box_y);
-		cairo_rotate (cr, -3.1415927 / 2.0);
-		cairo_show_text (cr, graph->priv->axis_label_y);
-		cairo_restore (cr);
-	}
-
 	gpm_graph_widget_draw_line (graph, cr);
 
+	if (graph->priv->use_events) {
+		gpm_graph_widget_draw_event_dots (graph, cr);
+	}
+
 	if (graph->priv->use_legend) {
 		gpm_graph_widget_draw_legend (graph, legend_x, legend_y, legend_width, legend_height);
 	}
