@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
- * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2005-2007 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -26,9 +26,9 @@
 #include <glib-object.h>
 #include <dbus/dbus-glib.h>
 
-#define	GPM_DBUS_SERVICE		"org.gnome.PowerManager"
-#define	GPM_DBUS_PATH			"/org/gnome/PowerManager"
-#define	GPM_DBUS_INTERFACE		"org.gnome.PowerManager"
+#define	GPM_DBUS_SERVICE		"org.freedesktop.PowerManagement"
+#define	GPM_DBUS_PATH			"/org/freedesktop/PowerManagement"
+#define	GPM_DBUS_INTERFACE		"org.freedesktop.PowerManagement"
 
 G_BEGIN_DECLS
 
@@ -38,6 +38,8 @@ G_BEGIN_DECLS
 #define GPM_IS_MANAGER(o)	 (G_TYPE_CHECK_INSTANCE_TYPE ((o), GPM_TYPE_MANAGER))
 #define GPM_IS_MANAGER_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), GPM_TYPE_MANAGER))
 #define GPM_MANAGER_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GPM_TYPE_MANAGER, GpmManagerClass))
+#define GPM_MANAGER_ERROR	 (gpm_manager_error_quark ())
+#define GPM_MANAGER_TYPE_ERROR	 (gpm_manager_error_get_type ()) 
 
 typedef struct GpmManagerPrivate GpmManagerPrivate;
 
@@ -50,56 +52,64 @@ typedef struct
 typedef struct
 {
 	GObjectClass	parent_class;
-	void		(* on_ac_changed)		(GpmManager	*manager,
-							 gboolean	 on_ac);
+	void		(* on_battery_changed)		(GpmManager	*manager,
+							 gboolean	 on_battery);
+	void		(* low_battery_changed)		(GpmManager	*manager,
+							 gboolean	 low_battery);
+	void		(* power_save_status_changed)	(GpmManager	*manager,
+							 gboolean	 save_power);
+	void		(* can_suspend_changed)		(GpmManager	*manager,
+							 gboolean	 can_suspend);
+	void		(* can_hibernate_changed)	(GpmManager	*manager,
+							 gboolean	 can_hibernate);
+	void		(* can_shutdown_changed)	(GpmManager	*manager,
+							 gboolean	 can_shutdown);
+	void		(* can_reboot_changed)		(GpmManager	*manager,
+							 gboolean	 can_reboot);
 } GpmManagerClass;
 
 typedef enum
 {
-	 GPM_MANAGER_ERROR_GENERAL
+	GPM_MANAGER_ERROR_DENIED,
+	GPM_MANAGER_ERROR_NO_HW,
+	GPM_MANAGER_ERROR_LAST
 } GpmManagerError;
 
-#define GPM_MANAGER_ERROR gpm_manager_error_quark ()
 
 GQuark		 gpm_manager_error_quark		(void);
+GType		 gpm_manager_error_get_type		(void);
 GType		 gpm_manager_get_type		  	(void);
 GpmManager	*gpm_manager_new			(void);
 
-gboolean	 gpm_manager_get_on_ac			(GpmManager	*manager,
-							 gboolean	*retval,
+gboolean	 gpm_manager_get_power_save_status	(GpmManager	*manager,
+							 gboolean	*save_power,
 							 GError		**error);
-gboolean	 gpm_manager_get_low_power_mode		(GpmManager	*manager,
-							 gboolean	*retval,
+gboolean	 gpm_manager_get_on_battery		(GpmManager	*manager,
+							 gboolean	*on_battery,
 							 GError		**error);
-gboolean	 gpm_manager_allowed_shutdown		(GpmManager	*manager,
-							 gboolean	*allowed,
+gboolean	 gpm_manager_get_low_battery		(GpmManager	*manager,
+							 gboolean	*low_battery,
 							 GError		**error);
-gboolean	 gpm_manager_allowed_reboot		(GpmManager	*manager,
-							 gboolean	*allowed,
+gboolean	 gpm_manager_suspend			(GpmManager	*manager,
 							 GError		**error);
-gboolean	 gpm_manager_allowed_suspend		(GpmManager	*manager,
-							 gboolean	*allowed,
+gboolean	 gpm_manager_hibernate			(GpmManager	*manager,
 							 GError		**error);
-gboolean	 gpm_manager_allowed_hibernate		(GpmManager	*manager,
-							 gboolean	*allowed,
+gboolean	 gpm_manager_reboot			(GpmManager	*manager,
 							 GError		**error);
-gboolean	 gpm_manager_suspend_dbus_method	(GpmManager	*manager,
+gboolean	 gpm_manager_shutdown			(GpmManager	*manager,
 							 GError		**error);
-gboolean	 gpm_manager_hibernate_dbus_method	(GpmManager	*manager,
+gboolean	 gpm_manager_can_suspend		(GpmManager	*manager,
+							 gboolean	*can_suspend,
 							 GError		**error);
-gboolean	 gpm_manager_shutdown_dbus_method	(GpmManager	*manager,
-						 	 GError		**error);
-gboolean	 gpm_manager_reboot_dbus_method		(GpmManager	*manager,
-						 	 GError		**error);
-void		 gpm_manager_inhibit			(GpmManager	*manager,
-						   	 const gchar	*application,
-							 const gchar	*reason,
-							 DBusGMethodInvocation *context,
+gboolean	 gpm_manager_can_hibernate		(GpmManager	*manager,
+							 gboolean	*can_hibernate,
+							 GError		**error);
+gboolean	 gpm_manager_can_reboot			(GpmManager	*manager,
+							 gboolean	*can_reboot,
+							 GError		**error);
+gboolean	 gpm_manager_can_shutdown		(GpmManager	*manager,
+							 gboolean	*can_shutdown,
 							 GError		**error);
-void		 gpm_manager_uninhibit			(GpmManager	*manager,
-						 	 guint32	 cookie,
-				   			 DBusGMethodInvocation *context,
-				   			 GError		**error);
 
 G_END_DECLS
 
