@@ -39,13 +39,14 @@
 #include <glib/gi18n.h>
 #include <dbus/dbus-glib.h>
 #include <gtk/gtk.h>
+#include <libgpm.h>
 
 #include "gpm-ac-adapter.h"
 #include "gpm-common.h"
 #include "gpm-conf.h"
 #include "gpm-debug.h"
-#include "gpm-power.h"
 #include "gpm-notify.h"
+#include "gpm-stock-icons.h"
 
 #ifdef HAVE_LIBNOTIFY
 #include <libnotify/notify.h>
@@ -57,9 +58,9 @@ struct GpmNotifyPrivate
 {
 	GpmAcAdapter		*ac_adapter;
 	GpmConf			*conf;
-	GpmPower		*power;
 	GtkStatusIcon		*status_icon;
 	gchar			*recall_website;
+	const gchar		*do_not_show_gconf;
 #ifdef HAVE_LIBNOTIFY
 	NotifyNotification	*libnotify;
 #endif
@@ -70,7 +71,8 @@ enum {
 	LAST_SIGNAL
 };
 
-//static guint	     signals [LAST_SIGNAL] = { 0, };
+//static guint signals [LAST_SIGNAL] = { 0, };
+static gpointer gpm_notify_object = NULL;
 
 G_DEFINE_TYPE (GpmNotify, gpm_notify, G_TYPE_OBJECT)
 
@@ -86,7 +88,7 @@ notify_closed_cb (NotifyNotification *libnotify,
 {
 	/* just invalidate the pointer */
 	gpm_debug ("caught notification closed signal");
-	notify->priv->libnotify = NULL;
+//	notify->priv->libnotify = NULL;
 }
 
 static gboolean
@@ -98,12 +100,10 @@ gpm_notify_create (GpmNotify 	 *notify,
 		   GpmNotifyUrgency urgency)
 {
 	if (notify->priv->libnotify != NULL) {
-		notify_notification_close (notify->priv->libnotify, NULL);
-		notify->priv->libnotify = NULL;
+//		notify_notification_close (notify->priv->libnotify, NULL);
+//		notify->priv->libnotify = NULL;
 	}
 
-/* DISTROS: If you've patched your libnotify 0.4.2 package you can add this define */
-#ifdef HAVE_LIBNOTIFY_NEW
 	if (notify->priv->status_icon != NULL &&
 	    gtk_status_icon_get_visible (notify->priv->status_icon) == TRUE) {
 		notify->priv->libnotify = notify_notification_new_with_status_icon (title, content,
@@ -112,16 +112,13 @@ gpm_notify_create (GpmNotify 	 *notify,
 	} else {
 		notify->priv->libnotify = notify_notification_new (title, content, msgicon, NULL);
 	}
-#else
-	/* we can't point because of a bug in libnotify. Need dependency on 0.4.3 */
-	notify->priv->libnotify = notify_notification_new (title, content, msgicon, NULL);
-#endif
+
 	if (timeout == GPM_NOTIFY_TIMEOUT_NEVER) {
 		notify_notification_set_timeout (notify->priv->libnotify, 0);
 	} else if (timeout == GPM_NOTIFY_TIMEOUT_LONG) {
-		notify_notification_set_timeout (notify->priv->libnotify, 20 * 1000);
+		notify_notification_set_timeout (notify->priv->libnotify, 30 * 1000);
 	} else if (timeout == GPM_NOTIFY_TIMEOUT_SHORT) {
-		notify_notification_set_timeout (notify->priv->libnotify, 5 * 1000);
+		notify_notification_set_timeout (notify->priv->libnotify, 10 * 1000);
 	}
 
 	if (urgency == GPM_NOTIFY_URGENCY_CRITICAL) {
@@ -172,6 +169,34 @@ gpm_notify_display (GpmNotify 	 *notify,
 
 #else
 
+/**
+ * gpm_notify_show:
+ *
+ * Stub.
+ **/
+static gboolean
+gpm_notify_show (GpmNotify *notify)
+{
+	return TRUE;
+}
+
+/**
+ * gpm_notify_create:
+ *
+ * Stub.
+ **/
+gboolean
+gpm_notify_create (GpmNotify 	 *notify,
+	 	   const gchar	 *title,
+		   const gchar	 *content,
+		   GpmNotifyTimeout timeout,
+		   const gchar	 *msgicon,
+		   GpmNotifyUrgency urgency)
+{
+	gpm_notify_display (notify, title, content, timeout, msgicon, urgency);
+	return TRUE;
+}
+
 /**
  * gpm_notify_display:
  * @notify: This class instance
@@ -250,13 +275,13 @@ gpm_notify_cancel (GpmNotify *notify)
  * Does the actions when the ac power source is inserted/removed.
  **/
 static void
-ac_adapter_changed_cb (GpmAcAdapter     *ac_adapter,
-		       GpmAcAdapterState state,
-		       GpmNotify        *notify)
+ac_adapter_changed_cb (GpmAcAdapter *ac_adapter,
+		       gboolean      on_ac,
+		       GpmNotify    *notify)
 {
 	/* for where we add back the ac_adapter before the "AC Power unplugged"
 	 * message times out. */
-	if (state == GPM_AC_ADAPTER_PRESENT) {
+	if (on_ac == TRUE) {
 		gpm_debug ("clearing notify due ac being present");
 		gpm_notify_cancel (notify);
 	}
@@ -275,14 +300,21 @@ gpm_notify_use_status_icon (GpmNotify *notify, GtkStatusIcon *status_icon)
 
 #ifdef HAVE_LIBNOTIFY
 static void
-notify_recall_action_cb (NotifyNotification *libnotify,
-                         gchar *action, GpmNotify *notify)
+notify_general_clicked_cb (NotifyNotification *libnotify,
+                           gchar *action, GpmNotify *notify)
 {
-	GError *error = NULL;
 	gboolean ret;
+	GError *error;
 
+	if (strcmp (action, "dont-show-again") == 0) {
+		gpm_debug ("not showing warning anymore for %s!", notify->priv->do_not_show_gconf);
+		gpm_conf_set_bool (notify->priv->conf, notify->priv->do_not_show_gconf, FALSE);
+		notify->priv->do_not_show_gconf = NULL;
+		return;
+	}
 	if (strcmp (action, "visit-website") == 0) {
 		gpm_debug ("autovisit website %s", notify->priv->recall_website);
+		error = NULL;
 		ret = gnome_url_show (notify->priv->recall_website, &error);
 		if (ret == FALSE) {
 			gpm_debug ("failed to show url: %s", error->message);
@@ -291,41 +323,24 @@ notify_recall_action_cb (NotifyNotification *libnotify,
 		/* free the stored string */
 		g_free (notify->priv->recall_website);
 		notify->priv->recall_website = NULL;
-	} else if (strcmp (action, "dont-show-again") == 0) {
-		gpm_debug ("not showing warning anymore!");
-		gpm_conf_set_bool (notify->priv->conf,
-				   GPM_CONF_SHOW_BATTERY_WARNING, FALSE);
+		return;
 	}
+	gpm_debug ("action %s unknown", action);
 }
+#endif
 
 /**
- * power_perhaps_recall_cb:
- * @power: The power class instance
- * @vendor: The battery vendor, e.g. "DELL"
- * @manager: This class instance
- *
- * This function splits up the battery status changed callback, and calls
- * different functions for each of the device types.
+ * gpm_notify_perhaps_recall:
  **/
-static void
-power_perhaps_recall_cb (GpmPower    *power,
-			 const gchar *oem_vendor,
-			 const gchar *website,
-			 GpmNotify   *notify)
+gboolean
+gpm_notify_perhaps_recall (GpmNotify   *notify,
+			   const gchar *oem_vendor,
+			   const gchar *website)
 {
 	gchar *msg;
 	const gchar *title;
 
-	/* check to see if HAL has given us all the right info */
-	if (oem_vendor == NULL || website == NULL) {
-		gpm_warning ("Possibly a potential critical hardware problem, "
-			     "but not enough data from HAL to report to the user");
-		return;
-	}
-
 	/* save in state */
-	notify->priv->recall_website = g_strdup (website);
-
 	title = _("Battery may be recalled");
 	msg = g_strdup_printf (_("The battery in your computer may have been "
 			       "recalled by %s and you may be "
@@ -338,21 +353,186 @@ power_perhaps_recall_cb (GpmPower    *power,
 			   GPM_NOTIFY_URGENCY_CRITICAL);
 
 	/* add extra stuff */
+#ifdef HAVE_LIBNOTIFY
+	notify->priv->recall_website = g_strdup (website);
 	notify_notification_add_action  (notify->priv->libnotify,
 	                                 "visit-website",
 	                                 _("Visit recall website"),
-	                                 (NotifyActionCallback) notify_recall_action_cb,
+	                                 (NotifyActionCallback) notify_general_clicked_cb,
 	                                 notify, NULL);
+	notify->priv->do_not_show_gconf = GPM_CONF_NOTIFY_PERHAPS_RECALL;
 	notify_notification_add_action  (notify->priv->libnotify,
 	                                 "dont-show-again",
 	                                 _("Do not show me this again"),
-	                                 (NotifyActionCallback) notify_recall_action_cb,
+	                                 (NotifyActionCallback) notify_general_clicked_cb,
 	                                 notify, NULL);
+#endif
+
 	gpm_notify_show (notify);
 	g_free (msg);
+	return TRUE;
 }
+
+/**
+ * gpm_notify_low_capacity:
+ **/
+gboolean
+gpm_notify_low_capacity (GpmNotify *notify,
+			 guint      capacity)
+{
+	gchar *msg;
+	const gchar *title;
+
+	title = _("Battery may be invalid");
+	msg = g_strdup_printf (_("Your battery has a very low capacity (%i%%), "
+				 "which means that it may be old or broken."),
+			       capacity);
+
+	gpm_notify_create (notify, title, msg, GPM_NOTIFY_TIMEOUT_LONG,
+			   GTK_STOCK_DIALOG_WARNING,
+			   GPM_NOTIFY_URGENCY_CRITICAL);
+
+	/* add extra stuff */
+#ifdef HAVE_LIBNOTIFY
+	notify->priv->do_not_show_gconf = GPM_CONF_NOTIFY_LOW_CAPACITY;
+	notify_notification_add_action  (notify->priv->libnotify,
+	                                 "dont-show-again",
+	                                 _("Do not show me this again"),
+	                                 (NotifyActionCallback) notify_general_clicked_cb,
+	                                 notify, NULL);
 #endif
 
+	gpm_notify_show (notify);
+	g_free (msg);
+	return TRUE;
+}
+
+/**
+ * gpm_notify_fully_charged_primary:
+ **/
+gboolean
+gpm_notify_fully_charged_primary (GpmNotify *notify)
+{
+	const gchar *msg;
+	const gchar *title;
+
+	title = _("Battery Charged");
+	msg = _("Your laptop battery is now fully charged");
+
+	gpm_notify_create (notify, title, msg, GPM_NOTIFY_TIMEOUT_SHORT,
+			   GTK_STOCK_DIALOG_WARNING,
+			   GPM_NOTIFY_URGENCY_CRITICAL);
+
+	/* add extra stuff */
+#ifdef HAVE_LIBNOTIFY
+	notify->priv->do_not_show_gconf = GPM_CONF_NOTIFY_FULLY_CHARGED;
+	notify_notification_add_action  (notify->priv->libnotify,
+	                                 "dont-show-again",
+	                                 _("Do not show me this again"),
+	                                 (NotifyActionCallback) notify_general_clicked_cb,
+	                                 notify, NULL);
+#endif
+
+	gpm_notify_show (notify);
+	return TRUE;
+}
+
+/**
+ * gpm_notify_discharging_primary:
+ **/
+gboolean
+gpm_notify_discharging_primary (GpmNotify *notify)
+{
+	const gchar *msg;
+	const gchar *title;
+
+	title = _("Battery Discharging");
+	msg = _("The AC power has been unplugged. The system is now using battery power.");
+
+	gpm_notify_create (notify, title, msg, GPM_NOTIFY_TIMEOUT_SHORT,
+			   GTK_STOCK_DIALOG_WARNING,
+			   GPM_NOTIFY_URGENCY_CRITICAL);
+
+	/* add extra stuff */
+#ifdef HAVE_LIBNOTIFY
+	notify->priv->do_not_show_gconf = GPM_CONF_NOTIFY_DISCHARGING;
+	notify_notification_add_action  (notify->priv->libnotify,
+	                                 "dont-show-again",
+	                                 _("Do not show me this again"),
+	                                 (NotifyActionCallback) notify_general_clicked_cb,
+	                                 notify, NULL);
+#endif
+
+	gpm_notify_show (notify);
+	return TRUE;
+}
+
+/**
+ * gpm_notify_discharging_ups:
+ **/
+gboolean
+gpm_notify_discharging_ups (GpmNotify *notify)
+{
+	const gchar *msg;
+	const gchar *title;
+
+	title = _("UPS Discharging");
+	msg = _("The AC power has been unplugged. The system is now using backup power.");
+
+	gpm_notify_create (notify, title, msg, GPM_NOTIFY_TIMEOUT_SHORT,
+			   GTK_STOCK_DIALOG_WARNING,
+			   GPM_NOTIFY_URGENCY_CRITICAL);
+
+	/* add extra stuff */
+#ifdef HAVE_LIBNOTIFY
+	notify->priv->do_not_show_gconf = GPM_CONF_NOTIFY_DISCHARGING;
+	notify_notification_add_action  (notify->priv->libnotify,
+	                                 "dont-show-again",
+	                                 _("Do not show me this again"),
+	                                 (NotifyActionCallback) notify_general_clicked_cb,
+	                                 notify, NULL);
+#endif
+
+	gpm_notify_show (notify);
+	return TRUE;
+}
+
+/**
+ * gpm_notify_sleep_failed:
+ **/
+gboolean
+gpm_notify_sleep_failed (GpmNotify *notify, gboolean hibernate)
+{
+	const gchar *msg;
+	const gchar *title;
+	const gchar *icon;
+
+	title = _("Sleep Problem");
+	if (hibernate == TRUE) {
+		msg = _("Your computer failed to hibernate.\nCheck the help file for common problems.");
+		icon = GPM_STOCK_HIBERNATE;
+	} else {
+		msg = _("Your computer failed to suspend.\nCheck the help file for common problems.");
+		icon = GPM_STOCK_SUSPEND;
+	}
+
+	gpm_notify_create (notify, title, msg, GPM_NOTIFY_TIMEOUT_LONG, icon,
+			   GPM_NOTIFY_URGENCY_CRITICAL);
+
+	/* add extra stuff */
+#ifdef HAVE_LIBNOTIFY
+	notify->priv->do_not_show_gconf = GPM_CONF_NOTIFY_SLEEP_FAILED;
+	notify_notification_add_action  (notify->priv->libnotify,
+	                                 "dont-show-again",
+	                                 _("Do not show me this again"),
+	                                 (NotifyActionCallback) notify_general_clicked_cb,
+	                                 notify, NULL);
+#endif
+
+	gpm_notify_show (notify);
+	return TRUE;
+}
+
 /**
  * gpm_notify_constructor:
  **/
@@ -379,19 +559,18 @@ gpm_notify_finalize (GObject *object)
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GPM_IS_NOTIFY (object));
 	notify = GPM_NOTIFY (object);
+	g_return_if_fail (notify->priv != NULL);
 
 #ifdef HAVE_LIBNOTIFY
 	if (notify->priv->libnotify != NULL) {
 		notify_notification_close (notify->priv->libnotify, NULL);
 	}
 #endif
-	g_object_unref (notify->priv->power);
 	g_object_unref (notify->priv->conf);
 	if (notify->priv->ac_adapter != NULL) {
 		g_object_unref (notify->priv->ac_adapter);
 	}
 
-	g_return_if_fail (notify->priv != NULL);
 	G_OBJECT_CLASS (gpm_notify_parent_class)->finalize (object);
 }
 
@@ -422,11 +601,7 @@ gpm_notify_init (GpmNotify *notify)
 	notify->priv = GPM_NOTIFY_GET_PRIVATE (notify);
 
 	notify->priv->conf = gpm_conf_new ();
-	notify->priv->power = gpm_power_new ();
-#ifdef HAVE_LIBNOTIFY
-	g_signal_connect (notify->priv->power, "battery-perhaps-recall",
-			  G_CALLBACK (power_perhaps_recall_cb), notify);
-#endif
+	notify->priv->do_not_show_gconf = NULL;
 
 	/* we use ac_adapter so we can log the event */
 	notify->priv->ac_adapter = gpm_ac_adapter_new ();
@@ -447,13 +622,11 @@ gpm_notify_init (GpmNotify *notify)
 GpmNotify *
 gpm_notify_new (void)
 {
-	static GpmNotify *notify = NULL;
-
-	if (notify != NULL) {
-		g_object_ref (notify);
-		return notify;
+	if (gpm_notify_object != NULL) {
+		g_object_ref (gpm_notify_object);
+	} else {
+		gpm_notify_object = g_object_new (GPM_TYPE_NOTIFY, NULL);
+		g_object_add_weak_pointer (gpm_notify_object, &gpm_notify_object);
 	}
-
-	notify = g_object_new (GPM_TYPE_NOTIFY, NULL);
-	return notify;
+	return GPM_NOTIFY (gpm_notify_object);
 }
