@@ -2,7 +2,7 @@
  *
  * Copyright (C) 2005 Jaap Haitsma <jaap@haitsma.org>
  * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
- * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2005-2007 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -32,17 +32,18 @@
 #include <math.h>
 #include <string.h>
 
+#include <libhal-gmanager.h>
+#include <libhal-gcpufreq.h>
+#include <libgpm.h>
+
 #include "gpm-tray-icon.h"
 #include "gpm-common.h"
 #include "gpm-prefs.h"
 #include "gpm-conf.h"
-#include "gpm-hal.h"
-#include "gpm-cpufreq.h"
 #include "gpm-prefs-core.h"
 #include "gpm-debug.h"
 #include "gpm-stock-icons.h"
 #include "gpm-screensaver.h"
-#include "gpm-powermanager.h"
 
 static void     gpm_prefs_class_init (GpmPrefsClass *klass);
 static void     gpm_prefs_init       (GpmPrefs      *prefs);
@@ -62,9 +63,8 @@ struct GpmPrefsPrivate
 	gboolean		 can_hibernate;
 	GpmConf			*conf;
 	GpmScreensaver		*screensaver;
-	GpmCpuFreq		*cpufreq;
-	GpmCpuFreqEnum		 cpufreq_types;
-	GpmHal			*hal;
+	HalGCpufreq		*hal_cpufreq;
+	HalGCpufreqType		 cpufreq_types;
 };
 
 enum {
@@ -86,6 +86,7 @@ G_DEFINE_TYPE (GpmPrefs, gpm_prefs, G_TYPE_OBJECT)
 #define ACTION_NOTHING_TEXT		_("Do nothing")
 
 /* The text that should appear in the processor combo box */
+#define CPUFREQ_NOTHING_TEXT		_("Do nothing")
 #define CPUFREQ_ONDEMAND_TEXT		_("Based on processor load")
 #define CPUFREQ_CONSERVATIVE_TEXT	_("Automatic power saving")
 #define CPUFREQ_POWERSAVE_TEXT		_("Maximum power saving")
@@ -483,8 +484,7 @@ gpm_prefs_setup_action_combo (GpmPrefs     *prefs,
 						   ACTION_NOTHING_TEXT);
 			n_added++;
 		} else {
-			gpm_critical_error ("Unknown action read from conf: %s",
-					    actions[i]);
+			gpm_error ("Unknown action read from conf: %s", actions[i]);
 		}
 
 		if (strcmp (value, actions[i]) == 0)
@@ -637,8 +637,8 @@ conf_key_changed_cb (GpmConf     *conf,
 		gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_UPS_LOWPOWER, &enabled);
 		gpm_debug ("need to enable checkbox");
 
-	} else if (strcmp (key, GPM_CONF_BATTERY_LOWPOWER) == 0) {
-		gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_BATTERY_LOWPOWER, &enabled);
+	} else if (strcmp (key, GPM_CONF_BATT_LOWPOWER) == 0) {
+		gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_BATT_LOWPOWER, &enabled);
 		gpm_debug ("need to enable checkbox");
 	}
 }
@@ -669,6 +669,8 @@ gpm_prefs_processor_combo_changed_cb (GtkWidget *widget,
 		policy = CODE_CPUFREQ_POWERSAVE;
 	} else if (strcmp (value, CPUFREQ_PERFORMANCE_TEXT) == 0) {
 		policy = CODE_CPUFREQ_PERFORMANCE;
+	} else if (strcmp (value, CPUFREQ_NOTHING_TEXT) == 0) {
+		policy = CODE_CPUFREQ_NOTHING;
 	} else {
 		g_assert (FALSE);
 	}
@@ -690,14 +692,14 @@ static void
 gpm_prefs_setup_processor_combo (GpmPrefs         *prefs,
 				 const gchar      *widget_name,
 				 const gchar      *gpm_pref_key,
-				 GpmCpuFreqEnum cpufreq_types)
+				 HalGCpufreqType cpufreq_types)
 {
 	gchar *value;
 	guint n_added = 0;
 	gboolean has_option = FALSE;
 	gboolean is_writable;
 	GtkWidget *widget;
-	GpmCpuFreqEnum cpufreq_type;
+	HalGCpufreqType cpufreq_type;
 
 	widget = glade_xml_get_widget (prefs->priv->glade_xml, widget_name);
 	gpm_conf_get_string (prefs->priv->conf, gpm_pref_key, &value);
@@ -715,46 +717,55 @@ gpm_prefs_setup_processor_combo (GpmPrefs         *prefs,
 			  G_CALLBACK (gpm_prefs_processor_combo_changed_cb),
 			  prefs);
 
-	cpufreq_type = gpm_cpufreq_string_to_enum (value);
+	cpufreq_type = hal_gcpufreq_string_to_enum (value);
 
-	if (cpufreq_types & GPM_CPUFREQ_ONDEMAND) {
+	if (cpufreq_types & LIBHAL_CPUFREQ_ONDEMAND) {
 		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 					   CPUFREQ_ONDEMAND_TEXT);
-		if (cpufreq_type == GPM_CPUFREQ_ONDEMAND) {
+		if (cpufreq_type == LIBHAL_CPUFREQ_ONDEMAND) {
 			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
 			has_option = TRUE;
 		}
 		n_added++;
 	}
-	if (cpufreq_types & GPM_CPUFREQ_CONSERVATIVE) {
+	if (cpufreq_types & LIBHAL_CPUFREQ_NOTHING) {
+		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
+					   CPUFREQ_NOTHING_TEXT);
+		if (cpufreq_type == LIBHAL_CPUFREQ_ONDEMAND) {
+			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
+			has_option = TRUE;
+		}
+		n_added++;
+	}
+	if (cpufreq_types & LIBHAL_CPUFREQ_CONSERVATIVE) {
 		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 					   CPUFREQ_CONSERVATIVE_TEXT);
-		if (cpufreq_type == GPM_CPUFREQ_CONSERVATIVE) {
+		if (cpufreq_type == LIBHAL_CPUFREQ_CONSERVATIVE) {
 			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
 			has_option = TRUE;
 		}
 		n_added++;
 	}
-	if (cpufreq_types & GPM_CPUFREQ_POWERSAVE) {
+	if (cpufreq_types & LIBHAL_CPUFREQ_POWERSAVE) {
 		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 					   CPUFREQ_POWERSAVE_TEXT);
-		if (cpufreq_type == GPM_CPUFREQ_POWERSAVE) {
+		if (cpufreq_type == LIBHAL_CPUFREQ_POWERSAVE) {
 			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
 			has_option = TRUE;
 		}
 		n_added++;
 	}
-	if (cpufreq_types & GPM_CPUFREQ_PERFORMANCE) {
+	if (cpufreq_types & LIBHAL_CPUFREQ_PERFORMANCE) {
 		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 					   CPUFREQ_PERFORMANCE_TEXT);
-		if (cpufreq_type == GPM_CPUFREQ_PERFORMANCE) {
+		if (cpufreq_type == LIBHAL_CPUFREQ_PERFORMANCE) {
 			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
 			has_option = TRUE;
 		}
 		n_added++;
 	}
 
-	if (has_option == FALSE || cpufreq_type == GPM_CPUFREQ_NOTHING) {
+	if (has_option == FALSE || cpufreq_type == LIBHAL_CPUFREQ_NOTHING) {
 		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
 	}
 	g_free (value);
@@ -854,12 +865,14 @@ prefs_setup_ac (GpmPrefs *prefs)
 {
 	GtkWidget *widget;
 	gint delay;
+	gboolean show_cpufreq;
 	const gchar  *button_lid_actions[] =
 				{ACTION_NOTHING,
 				 ACTION_BLANK,
 				 ACTION_SUSPEND,
 				 ACTION_HIBERNATE,
 				 NULL};
+
 	gpm_prefs_setup_action_combo (prefs, "combobox_ac_lid",
 				      GPM_CONF_AC_BUTTON_LID,
 				      button_lid_actions);
@@ -880,7 +893,7 @@ prefs_setup_ac (GpmPrefs *prefs)
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_ac_lid");
 		gtk_widget_hide_all (widget);
 	}
-	if (prefs->priv->cpufreq == NULL) {
+	if (prefs->priv->hal_cpufreq == NULL) {
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_ac_cpu");
 		gtk_widget_hide_all (widget);
 	}
@@ -888,6 +901,12 @@ prefs_setup_ac (GpmPrefs *prefs)
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_ac_brightness");
 		gtk_widget_hide_all (widget);
 	}
+
+	gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_UI_SHOW_CPUFREQ, &show_cpufreq);
+	if (show_cpufreq == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_ac_cpu");
+		gtk_widget_hide_all (widget);
+	}
 }
 
 static void
@@ -897,7 +916,8 @@ prefs_setup_battery (GpmPrefs *prefs)
 	GtkWidget *notebook;
 	gint delay;
 	gint page;
-	
+	gboolean show_cpufreq;
+
 	const gchar  *button_lid_actions[] =
 				{ACTION_NOTHING,
 				 ACTION_BLANK,
@@ -920,19 +940,19 @@ prefs_setup_battery (GpmPrefs *prefs)
 	}
 
 	gpm_prefs_setup_action_combo (prefs, "combobox_battery_lid",
-				      GPM_CONF_BATTERY_BUTTON_LID,
+				      GPM_CONF_BATT_BUTTON_LID,
 				      button_lid_actions);
 	gpm_prefs_setup_action_combo (prefs, "combobox_battery_critical",
-				      GPM_CONF_BATTERY_CRITICAL,
+				      GPM_CONF_BATT_CRITICAL,
 				      battery_critical_actions);
 	gpm_prefs_setup_processor_combo (prefs, "combobox_battery_cpu",
-					 GPM_CONF_BATTERY_CPUFREQ_POLICY, prefs->priv->cpufreq_types);
+					 GPM_CONF_BATT_CPUFREQ_POLICY, prefs->priv->cpufreq_types);
 	gpm_prefs_setup_sleep_slider (prefs, "hscale_battery_computer",
-				      GPM_CONF_BATTERY_SLEEP_COMPUTER);
+				      GPM_CONF_BATT_SLEEP_COMPUTER);
 	gpm_prefs_setup_sleep_slider (prefs, "hscale_battery_display",
-				      GPM_CONF_BATTERY_SLEEP_DISPLAY);
+				      GPM_CONF_BATT_SLEEP_DISPLAY);
 	gpm_prefs_setup_brightness_slider (prefs, "hscale_battery_brightness",
-					   GPM_CONF_BATTERY_BRIGHTNESS);
+					   GPM_CONF_BATT_BRIGHTNESS);
 
 	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
 	set_idle_hscale_stops (prefs, "hscale_battery_computer", delay);
@@ -942,7 +962,7 @@ prefs_setup_battery (GpmPrefs *prefs)
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_battery_lid");
 		gtk_widget_hide_all (widget);
 	}
-	if (prefs->priv->cpufreq == NULL) {
+	if (prefs->priv->hal_cpufreq == NULL) {
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_battery_cpu");
 		gtk_widget_hide_all (widget);
 	}
@@ -950,6 +970,11 @@ prefs_setup_battery (GpmPrefs *prefs)
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_battery_brightness");
 		gtk_widget_hide_all (widget);
 	}
+	gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_UI_SHOW_CPUFREQ, &show_cpufreq);
+	if (show_cpufreq == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_battery_cpu");
+		gtk_widget_hide_all (widget);
+	}
 }
 
 static void
@@ -981,7 +1006,7 @@ prefs_setup_ups (GpmPrefs *prefs)
 				      GPM_CONF_UPS_CRITICAL,
 				      ups_low_actions);
 	gpm_prefs_setup_sleep_slider (prefs, "hscale_ups_computer",
-				      GPM_CONF_BATTERY_SLEEP_COMPUTER);
+				      GPM_CONF_BATT_SLEEP_COMPUTER);
 	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
 	set_idle_hscale_stops (prefs, "hscale_ups_computer", delay);
 }
@@ -1027,13 +1052,13 @@ prefs_setup_general (GpmPrefs *prefs)
 static void
 gpm_prefs_init (GpmPrefs *prefs)
 {
-	GtkWidget    *main_window;
-	GtkWidget    *widget;
+	GtkWidget *main_window;
+	GtkWidget *widget;
+	HalGManager *hal_manager;
 
 	prefs->priv = GPM_PREFS_GET_PRIVATE (prefs);
 
-	prefs->priv->hal = gpm_hal_new ();
-	prefs->priv->cpufreq = gpm_cpufreq_new ();
+	prefs->priv->hal_cpufreq = hal_gcpufreq_new ();
 
 	prefs->priv->screensaver = gpm_screensaver_new ();
 	g_signal_connect (prefs->priv->screensaver, "gs-delay-changed",
@@ -1043,28 +1068,31 @@ gpm_prefs_init (GpmPrefs *prefs)
 	g_signal_connect (prefs->priv->conf, "value-changed",
 			  G_CALLBACK (conf_key_changed_cb), prefs);
 
-	prefs->priv->has_lcd = gpm_hal_num_devices_of_capability (prefs->priv->hal, "laptop_panel") > 0;
-	prefs->priv->has_batteries = gpm_hal_num_devices_of_capability_with_value (prefs->priv->hal, "battery",
+	hal_manager = hal_gmanager_new ();
+	prefs->priv->has_lcd = hal_gmanager_num_devices_of_capability (hal_manager, "laptop_panel") > 0;
+	prefs->priv->has_batteries = hal_gmanager_num_devices_of_capability_with_value (hal_manager, "battery",
 							"battery.type",
 							"primary") > 0;
-	prefs->priv->has_ups = gpm_hal_num_devices_of_capability_with_value (prefs->priv->hal, "battery",
+	prefs->priv->has_ups = hal_gmanager_num_devices_of_capability_with_value (hal_manager, "battery",
 							"battery.type",
 							"ups") > 0;
-	prefs->priv->has_button_lid = gpm_hal_num_devices_of_capability_with_value (prefs->priv->hal, "button",
+	prefs->priv->has_button_lid = hal_gmanager_num_devices_of_capability_with_value (hal_manager, "button",
 							"button.type",
 							"lid") > 0;
-	prefs->priv->has_button_suspend = gpm_hal_num_devices_of_capability_with_value (prefs->priv->hal, "button",
+	prefs->priv->has_button_suspend = hal_gmanager_num_devices_of_capability_with_value (hal_manager, "button",
 							"button.type",
 							"suspend") > 0;
-	prefs->priv->can_suspend = gpm_dbus_method_bool ("AllowedSuspend");
-	prefs->priv->can_hibernate = gpm_dbus_method_bool ("AllowedHibernate");
+	g_object_unref (hal_manager);
+
+	prefs->priv->can_suspend = gpm_dbus_method_bool ("CanSuspend");
+	prefs->priv->can_hibernate = gpm_dbus_method_bool ("CanHibernate");
 
 	/* only enable cpufreq stuff if we have the hardware */
-	if (prefs->priv->cpufreq) {
-		gpm_cpufreq_get_governors (prefs->priv->cpufreq,
-					       &prefs->priv->cpufreq_types);
+	if (prefs->priv->hal_cpufreq) {
+		hal_gcpufreq_get_governors (prefs->priv->hal_cpufreq,
+					    &prefs->priv->cpufreq_types);
 	} else {
-		prefs->priv->cpufreq_types = GPM_CPUFREQ_NOTHING;
+		prefs->priv->cpufreq_types = LIBHAL_CPUFREQ_NOTHING;
 	}
 
 	prefs->priv->glade_xml = glade_xml_new (GPM_DATA "/gpm-prefs.glade", NULL, NULL);
@@ -1118,11 +1146,8 @@ gpm_prefs_finalize (GObject *object)
 	if (prefs->priv->screensaver) {
 		g_object_unref (prefs->priv->screensaver);
 	}
-	if (prefs->priv->cpufreq) {
-		g_object_unref (prefs->priv->cpufreq);
-	}
-	if (prefs->priv->hal) {
-		g_object_unref (prefs->priv->hal);
+	if (prefs->priv->hal_cpufreq) {
+		g_object_unref (prefs->priv->hal_cpufreq);
 	}
 
 	G_OBJECT_CLASS (gpm_prefs_parent_class)->finalize (object);
